import os

mimes_map = {
    'txt': 'text/plain',
    'htm': 'text/html',
    'html': 'text/html',
    'php': 'text/html',
    'css': 'text/css',
    'js': 'application/javascript',
    'json': 'application/json',
    'xml': 'application/xml',
    'swf': 'application/x-shockwave-flash',
    'flv': 'video/x-flv',
    
    # images
    'png': 'image/png',
    'jpe': 'image/jpeg',
    'jpeg': 'image/jpeg',
    'jpg': 'image/jpeg',
    'gif': 'image/gif',
    'bmp': 'image/bmp',
    'ico': 'image/vnd.microsoft.icon',
    'tiff': 'image/tiff',
    'tif': 'image/tiff',
    'svg': 'image/svg+xml',
    'svgz': 'image/svg+xml',
    
    # archives
    'zip': 'application/zip',
    'rar': 'application/x-rar-compressed',
    'exe': 'application/x-msdownload',
    'msi': 'application/x-msdownload',
    'cab': 'application/vnd.ms-cab-compressed',
    
    # audio/video
    'mp3': 'audio/mpeg',
    'ogg': 'audio/ogg',
    'qt': 'video/quicktime',
    'mp4': 'video/mp4',
    'mov': 'video/quicktime',
    'wav': 'audio/x-wav',
    
    # adobe
    'pdf': 'application/pdf',
    'psd': 'image/vnd.adobe.photoshop',
    'ai': 'application/postscript',
    'eps': 'application/postscript',
    'ps': 'application/postscript',
    
    # ms office
    'doc': 'application/msword',
    'rtf': 'application/rtf',
    'xls': 'application/vnd.ms-excel',
    'ppt': 'application/vnd.ms-powerpoint',
    
    # open office
    'odt': 'application/vnd.oasis.opendocument.text',
    'ods': 'application/vnd.oasis.opendocument.spreadsheet',
}

exts_map = {v: k for k, v in mimes_map.items()}


def ext_to_mime(ext):
    """
    extension to mime
    :param ext: str
    :rtype: str
    """
    if ext in mimes_map.keys():
        return mimes_map[ext]
    else:
        return 'application/octet-stream'


def mime_to_ext(mime):
    """
    mime to extension
    :param mime:
    :return:
    """
    if mime in exts_map.keys():
        return exts_map[mime]
    else:
        return ''



from urllib.parse import urlparse, parse_qs


def parse_query(url):
    """
    get query dict of url
    :param url:
    :return:
    """
    result = urlparse(url)
    query = result.query
    query_dict = parse_qs(query)
    query_dict = {k: v[0] for k, v in query_dict.items()}
    return query_dict



from douyin.utils import fetch
from douyin.config import hot_trend_url, common_headers
from douyin.utils.tranform import data_to_music, data_to_topic
from douyin.structures.hot import HotTrend
from douyin.utils.common import parse_datetime


# define trend query params
query = {
    'version_code': '2.9.1',
    'count': '10',
}


def trend():
    """
    get trend result
    :return:
    """
    offset = 0
    while True:
        query['cursor'] = str(offset)
        result = fetch(hot_trend_url, headers=common_headers, params=query, verify=False)
        category_list = result.get('category_list')
        datetime = parse_datetime(result.get('extra', {}).get('now'))
        final = []
        for item in category_list:
            # process per category
            if item.get('desc') == '热门话题':
                final.append(data_to_topic(item.get('challenge_info', {})))
            if item.get('desc') == '热门音乐':
                final.append(data_to_music(item.get('music_info', {})))
        yield HotTrend(datetime=datetime, data=final, offset=offset, count=int(query.get('count')))
        offset += 10



# urls
hot_search_url = 'https://api.amemv.com/aweme/v1/hot/search/list/'
hot_video_url = 'https://api.amemv.com/aweme/v1/hotsearch/aweme/billboard/'
hot_energy_url = 'https://api.amemv.com/aweme/v1/hotsearch/positive_energy/billboard/'
hot_music_url = 'https://api.amemv.com/aweme/v1/hotsearch/music/billboard/'
hot_trend_url = 'https://api.amemv.com/aweme/v1/category/list/'
topic2video_url = 'https://api.amemv.com/aweme/v1/challenge/aweme/'
music2video_url = 'https://api.amemv.com/aweme/v1/music/aweme/'

# http
fetch_timeout = 5
common_headers = {
    'User-Agent': 'Aweme 2.9.1 rv:29101 (iPhone; iOS 12.0; zh_CN) Cronet',
}

# retrying
retry_max_number = 10
retry_min_random_wait = 1000  # ms
retry_max_random_wait = 5000  # ms



from douyin.config import hot_search_url
from douyin.structures import HotSearch
from douyin.utils import fetch


def search():
    """
    get hot search result
    :return: HotSearch object
    """
    result = fetch(hot_search_url)
    # process json data
    datetime = result.get('data', {}).get('active_time')
    word_list = result.get('data', {}).get('word_list', [])
    data = [{'item': item.get('word'), 'hot_value': item.get('hot_value')} for item in word_list]
    # construct HotSearch object and return
    return HotSearch(datetime=datetime, data=data)



from douyin.handlers import Handler
from motor.motor_asyncio import AsyncIOMotorClient
from douyin.structures import *


class MongoHandler(Handler):
    
    def __init__(self, conn_uri=None, db='douyin'):
        """
        init save folder
        :param folder:
        """
        super().__init__()
        if not conn_uri:
            conn_uri = 'localhost'
        self.client = AsyncIOMotorClient(conn_uri)
        self.db = self.client[db]
    
    async def process(self, obj, **kwargs):
        """
        download to file
        :param url: resource url
        :param name: save name
        :param kwargs:
        :return:
        """
        collection_name = 'default'
        if isinstance(obj, Video):
            collection_name = 'videos'
        elif isinstance(obj, Music):
            collection_name = 'musics'
        collection = self.db[collection_name]
        # save to mongodb
        print('Saving', obj, 'to mongodb...')
        if await collection.update_one({'id': obj.id}, {'$set': obj.json()}, upsert=True):
            print('Saved', obj, 'to mongodb successfully')
        else:
            print('Error occurred while saving', obj)



from douyin.structures import Base


class Video(Base):
    
    def __init__(self, **kwargs):
        """
        init video object
        :param kwargs:
        """
        super().__init__()
        self.id = kwargs.get('id')
        self.desc = kwargs.get('desc')
        self.author = kwargs.get('author')
        self.music = kwargs.get('music')
        self.like_count = kwargs.get('like_count')
        self.comment_count = kwargs.get('comment_count')
        self.share_count = kwargs.get('share_count')
        self.hot_count = kwargs.get('hot_count')
        self.play_url = kwargs.get('play_url')
        self.is_ads = kwargs.get('is_ads') or False
        self.duration = kwargs.get('duration')
        self.create_time = kwargs.get('create_time')
        self.share_url = kwargs.get('share_url')
        self.ratio = kwargs.get('ratio')
        self.cover_url = kwargs.get('cover_url')
        self.address = kwargs.get('address')
    
    def __repr__(self):
        """
        video to str
        :return: str
        """
        return '<Video: <%s, %s>>' % (self.id, self.desc[:10].strip() if self.desc else None)
    
    
    


class Handler(object):
    
    def process(self, obj, **kwargs):
        """
        parent method, you must implement this method in child object
        :param obj:
        :param kwargs:
        :return:
        """
        raise NotImplementedError
